
# q-expansion of newform 40.10.a.d, downloaded from the LMFDB on 28 April 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 40
weight = 10
poly_data = [-22, -19, -1, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0], 1], [[128, 84, -12], 1], [[-15488, -3144, 1272], 1]]

hecke_ring_character_values = None
aps_data = [[0, 0, 0], [28, 1, 0], [625, 0, 0], [-1840, 17, -1], [1852, 292, 1], [-27698, -28, -16], [122354, -1596, 0], [496372, -378, 81], [-166640, -1417, -319], [1744894, -6128, -380], [4236304, 6022, 238], [7241478, 50680, -1544], [9146826, 3188, -556], [-7739420, -162615, 1134], [-9567176, 115901, 2081], [-15209994, -175444, -5104], [-89573956, 183246, 5775], [51990046, -362224, 12800], [-175638868, 277043, -4270], [-79964808, -772366, 12320], [66120810, 460180, -4832], [-137946576, 298676, -29392], [123983276, -2169743, 21868], [251642202, 2366280, 20184], [-301150334, -1338708, -18648], [331105926, -2780080, -194020], [-133201856, -4798581, -89151], [-269075340, -9256133, -18368], [-785923186, -8596168, 125072], [-147036750, 2634696, 114072], [-1903505208, 2675917, 409309], [2912721876, -4943068, -288781], [-2902868918, 18875024, 169400], [1775576460, -7416378, 826371], [75004022, 2149800, 927936], [65422856, -21648478, -1131412], [-1867585730, 45307160, -642184], [5646945820, -6008275, 1052612], [-3462859824, 61337789, 1270211], [8014474190, 5286680, -537352], [5719279604, -25859822, -1334075], [12230102678, 52168072, -1305044], [10459751696, 15487058, 2198882], [6638047138, -35171436, 2963280], [11980865926, -61437572, 1727080], [-17220166456, -9709644, 2190114], [2141118308, -55445452, -496081], [-27800159944, 79038279, 2169759], [18333308364, -154115917, 3063002], [-23483901626, 202821416, 4302836], [-10921789878, 124653452, 297416], [27294101136, -147601828, -3738640], [-25569733294, 23139476, 4501388], [72495853356, 100481180, -2132131], [106032298050, 79137688, -430232], [95666505216, -94899281, 6512893], [13482931694, -640452256, 3720464], [-48200445216, -227167094, 5129206], [80606998262, 77727688, -7332416], [-32968748678, -205900836, -3113940], [2402057492, -203462935, -19419358], [31739399110, 238364000, 155528], [4808811676, -59167829, 14898958], [-52291570360, 180653078, 27382712], [-6437154886, -470590104, 12711120], [184115947902, -561681500, -8947160], [-229470304932, 459332112, -691443], [188391516178, -502881608, -12800912], [-67933351324, 1338375123, -4990356], [-109165488642, 798590712, -36163884], [-49245802846, -581533960, -53672944], [114159661800, 2050505076, -45420822], [-91680552040, 54260561, 69589061], [-100574618666, -879145256, -63032000], [177943515100, 921095290, 16678075], [298359002712, -1326455641, -11894857], [143980693542, 1495714144, 45225472], [178976664142, -1962918044, 11763328], [65113431570, -1811718608, -3100952], [-193945278854, 2413894908, -86058324], [-14823833820, 348429458, 31829225], [377375347974, 2974686728, -26652208], [602645084864, 1726784162, -68816986], [236014018514, 289802204, 146406680], [-410034560072, -1493286068, -106009970], [-145971005676, -284367979, 101681126], [-688294002270, 122063924, 93726020], [83258634634, 2996532432, 165520944], [-520195158738, 4242557968, -60077492], [19748175720, -3755521017, -7364253], [-690080368996, 2513792887, 33630850], [-176059952960, -3192528964, -27546172], [-361189000112, 820246085, -134989381], [-185898600804, -990564156, -18838311], [-506653050412, -367145642, 202653241], [106468540560, -340586517, -45638691], [-411623821538, -1128026008, -29989516], [970578308170, 4435072448, 221675168], [-1069688858876, -8758029471, 82419870], [1637303241598, 6191697472, -200339108], [541332767180, -7624571445, 235003266], [-1535199586354, -619092728, 97231168], [-2077601305076, -2729499311, 133531588], [-638134399846, -12658733716, -12917428], [-1315754213492, 5344508592, 95556609], [-647621845438, -6252078592, -499085512], [893316834996, 7300650631, -16348412], [248740690002, -4878971120, 209710912], [211322702712, 7799773072, 461583610], [-457607613638, 15193388236, 456098812], [479495934696, 14257135173, -226224003], [466663843686, 12947833684, -397289192], [-2274961363254, 1171577580, 499803336], [-1245458005492, -17399082470, 2455471], [-3172947077528, -22980766302, 468620916], [-3125921505438, 9180671572, -452272196], [-482346363204, 109046069, 259495076], [1917332105712, 3297225781, -659739869], [-25779462514, 6349823548, 540238408], [501721811412, -1121250814, -394010947], [3227509805622, 7175013488, 1001865440], [2744685988482, -18585366572, -21582920], [2676781985446, 3976220180, 129126392], [4527384120996, 5803309937, 96435638], [-207823171228, 28167918456, -741936165], [-2737492350370, -18489833768, 262818064], [7110345418214, -4046229296, -388780268], [-967361044880, 9321222348, 780688104], [-3228688500640, 3658270417, 928745515], [-992390405826, -34285967808, 343809864], [3173273056740, -906970718, -84782711], [-3973314620928, 2341268083, -1670878199], [-11085291006432, -15287214250, -261363334], [1700084644118, -7780475968, 608846336], [3818627577786, -15276960928, 929061968], [2030592454210, 1111200392, -42500440], [8253773592006, 12929625756, 764579880], [-7022636407972, -11133018585, 504221010], [-3211578357858, -2086152292, -684492952], [-1614909460374, -79356427392, 607674720], [905430657500, -87211617948, -888148359], [16618960984342, 2833259688, 97297008], [2853340165360, -20492699969, 824751169], [2835773456388, 46737180187, 232298332], [-7398229708514, -739702704, -562998720], [3685617871976, 14032281384, 111962022], [-3863440955242, 338969820, 207756360], [-4927640018214, -55657960216, -309684280], [10908038032796, 52650564246, -1553074929], [19027197801816, 23445138091, -532764797], [-4199942685362, 131060248352, 1801831208], [13210459477650, -26176172468, -1057847468], [-6485657785972, -4881115495, -302972548], [-1088825690048, 131764112937, -848056197], [1838494790964, -9708236273, 472711180], [-2479769996288, -93135852626, -1842722150], [12821353260920, 15699478120, -2919339134], [-12175422563966, -60635529900, -2114704236], [-14372800022966, -159889591388, 2790739768], [-2678049441842, 48312125080, 3437610076], [-989003602916, -7249763893, 4784168078], [3696731831802, 65293155440, 4608688064], [-13250050017072, 116283913977, 1502216919], [8035824145436, 87207110136, -396330891], [28251994542002, -63292105404, 288065736], [9436606179216, 76857205871, -3065871319], [-8815698404848, -4890992882, -1424425226], [-9912817488282, -51077058940, -6094180480]]
